/*
% A bump mapping shader.
author: Alex Lemke
date: 4/20/12
*/

#include "include\Common.fxh"

float4x4 WorldViewProj : WORLDVIEWPROJECTION < string UIWidget = "none"; >;
float4x4 WorldInverseTranspose : WORLDINVERSETRANSPOSE < string UIWidget = "none"; >;
float4x4 World : WORLD < string UIWidget = "none"; >;
float4x4 ViewInverse : VIEWINVERSE < string UIWidget = "none"; >;

float4 AmbientColor : COLOR < string UIName = "Ambient Color"; string UIWidget = "Color"; > = {0.1, 0.1, 0.1, 0.1};

//Specular
float4 SpecularColor : SPECULAR < string UIName = "Specular Color"; string UIWidget = "Color"; >;
float SpecularPower : SPECULARPOWER < string UIName = "Specular Power"; string UIWidget = "slider"; float UIMin = 1.0f, UIMax = 255.0f; float UIStep = 1.0f; >;

//Fog
float3 FogColor : COLOR < string UIWidget = "Fog Color"; > = {0.1, 0.1, 0.1};
float FogStart < string UIName = "Fog Start"; > = {10.0};
float FogEnd < string UIName = "Fog End"; > = {100.0};
bool FogEnabled < string UIName = "Fog Enabled"; > = false;

//Point Lights
#define NUMBER_OF_POINT_LIGHTS 4
struct PointLight
{
	float3 Position : POSITION;
	float4 Color : DIFFUSE;
	float Radius;
};
PointLight pointLights[NUMBER_OF_POINT_LIGHTS];

//Height Mapping
float DisplacementScale < string UIName = "Displacement Scale"; string UIWidget = "slider"; float UIMin = 0.0f, UIMax = 2.0f; float UIStep = 0.1f; > = {0.0f};



//Texture
texture ColorTexture;
sampler ColorTextureSampler : register(s0) = sampler_state
{
	Texture = (ColorTexture);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

texture NormalMap;
sampler NormalMapSampler : register(s1) = sampler_state
{
	Texture = (NormalMap);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

texture DisplacementMap;
sampler DisplacementMapSampler : register(s1) = sampler_state
{
	Texture = (DisplacementMap);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};


struct VS_IN
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
	float3 Normal 	: NORMAL;
	float3 Tangent  : TANGENT;
};

struct VS_OUT
{
	float4 Position 		: POSITION0;
	float2 TexCoord 		: TEXCOORD0;
	float3 ViewDirection	: TEXCOORD1;
	float FogAmount 		: TEXCOORD2;
	
	float4 LightDirection0 	: TEXCOORD3;
	float4 LightDirection1 	: TEXCOORD4;
	float4 LightDirection2	: TEXCOORD5;
	float4 LightDirection3 	: TEXCOORD6;
};

VS_OUT mainVS(VS_IN In)
{
	VS_OUT output;
	
	output.Position = mul(float4(In.Position.xyz, 1.0), WorldViewProj);
	output.TexCoord = get_corrected_texture_coord(In.TexCoord);
	
	float3x3 worldToTangentSpace = world_to_tangent_space(In.Tangent, In.Normal, World);
	float4 worldPosition = mul(float4(In.Position.xyz, 1.0), World);
	
	//Height Map
	float height = tex2Dlod(DisplacementMapSampler, float4(In.TexCoord.x, In.TexCoord.y, 0.0f, 0.0f)) - 0.5f;
	
	//output.Position.xyz += In.Normal * height * DisplacementScale;
	
	//Lights
	output.LightDirection0 = get_light_data(pointLights[0].Position, worldPosition, pointLights[0].Radius, worldToTangentSpace);
	output.LightDirection1 = get_light_data(pointLights[1].Position, worldPosition, pointLights[1].Radius, worldToTangentSpace);
	output.LightDirection2 = get_light_data(pointLights[2].Position, worldPosition, pointLights[2].Radius, worldToTangentSpace);
	output.LightDirection3 = get_light_data(pointLights[3].Position, worldPosition, pointLights[3].Radius, worldToTangentSpace);

	float3 cameraPosition = ViewInverse[3].xyz;
	
	output.ViewDirection = normalize(mul(worldToTangentSpace, cameraPosition - worldPosition));
	output.FogAmount = get_fog_amount(cameraPosition, worldPosition, FogStart, FogEnd);
		
	return output;
}

float4 mainPS(VS_OUT input) : COLOR 
{
	//Getting surface color
	float4 textureColor = tex2D(ColorTextureSampler, input.TexCoord);
	
	LightingCoefficientStruct lightData;

	//Normalizing Inputs
	lightData.Normal = (2 * (tex2D(NormalMapSampler,  input.TexCoord))) - 1.0;
	lightData.View = normalize(input.ViewDirection);
	lightData.SurfaceColor = textureColor;
	lightData.SpecularColor = SpecularColor;
	lightData.SpecularPower = SpecularPower;
	
	float4 output = float4(0,0,0,0);
	
	lightData.LightColor = pointLights[0].Color;
	lightData.LightDirection = input.LightDirection0;
	output += calculate_lighting(lightData);
	
	lightData.LightColor = pointLights[1].Color;
	lightData.LightDirection = input.LightDirection1;
	output += calculate_lighting(lightData);
	
	
	lightData.LightColor = pointLights[2].Color;
	lightData.LightDirection = input.LightDirection2;
	output += calculate_lighting(lightData);
	
	float4 ambient = get_color_contribution(AmbientColor, textureColor);
	
	//Adding ambient
	output += ambient;
	
	if(FogEnabled)
	{
		output.rgb = apply_fog(FogColor, output, input.FogAmount);
	}
	
	return output;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_3_0 mainVS();
		PixelShader = compile ps_3_0 mainPS();
	}
}
