/*
% A fog shader with ambient light, directional light and texture mapping.
author: Alex Lemke
date: 4/20/12
*/

#include "include\Common.fxh"

float4x4 WorldViewProj : WORLDVIEWPROJECTION < string UIWidget = "none"; >;
float4x4 WorldInverseTranspose : WORLDINVERSETRANSPOSE < string UIWidget = "none"; >;
float4x4 World : WORLD < string UIWidget = "none"; >;
float4x4 ViewInverse : VIEWINVERSE < string UIWidget = "none"; >;

float4 AmbientColor : COLOR < string UIName = "Ambient Color"; string UIWidget = "Color"; > = {0.1, 0.1, 0.1, 0.1};

float3 DirectionalLightDirection : DIRECTION < string Object = "DirectionalLight"; string UIName = "DirectionalLight 0 Direction"; string Space = "World"; >;
float4 DirectionalLightColor : DIFFUSE < string Object = "DirectionalLight"; string UIName = "DirectionalLight 0 Color"; >;

//Specular
float4 SpecularColor : SPECULAR < string UIName = "Specular Color"; string UIWidget = "Color"; >  = {0.7, 0.7, 0.7, 1.0};
float SpecularPower : SpecularPower < string UIName = "Specular Power"; string UIWidget = "slider"; float UIMin = 1.0f, UIMax = 128.0f; float UIStep = 1.0f; >;

//Fog
float3 FogColor : COLOR < string UIWidget = "Fog Color"; > = {0.1, 0.1, 0.1};
float FogStart < string UIName = "Fog Start"; > = {10.0};
float FogEnd < string UIName = "Fog End"; > = {100.0};
bool FogEnabled < string UIName = "Fog Enabled"; > = false;

texture ColorTexture;
sampler ColorTextureSampler : register(s0) = sampler_state
{
	Texture = (ColorTexture);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

struct VS_IN
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
	float3 Normal 	: NORMAL;
};

struct VS_OUT
{
	float4 Position 		: POSITION0;
	float2 TexCoord 		: TEXCOORD0;
	float3 LightDirection 	: TEXCOORD1;
	float3 Normal 			: TEXCOORD2;
	float3 ViewDirection	: TEXCOORD3;
	float FogAmount 		: TEXCOORD4;	 
};

VS_OUT mainVS(VS_IN input)
{
	VS_OUT output;
	
	output.Position = mul(float4(input.Position.xyz, 1.0), WorldViewProj);
	output.TexCoord = get_corrected_texture_coord(input.TexCoord);
	output.Normal = mul(float4(input.Normal.xyz, 1.0), WorldInverseTranspose);
	output.LightDirection = normalize(DirectionalLightDirection);
		
	float4 worldPosition = mul(float4(input.Position.xyz, 1.0), World);
	float3 cameraPosition = ViewInverse[3].xyz;
	
	output.ViewDirection = normalize(cameraPosition - worldPosition);
	output.FogAmount = get_fog_amount(cameraPosition, worldPosition, FogStart, FogEnd);
	
	return output;
}

float4 mainPS(VS_OUT input) : COLOR 
{
	//Normalizing Inputs
	float3 normal = normalize(input.Normal);
	
	//Light
	float3 lightDirection = -normalize(input.LightDirection);
	float NdotL = saturate(dot(lightDirection, normal));	
	
	float4 color = tex2D(ColorTextureSampler, input.TexCoord);
	float4 ambient = get_color_contribution(AmbientColor, color);
	float4 diffuse = get_color_contribution(DirectionalLightColor, NdotL * color);
	
	float4 specular = 0;
	if(NdotL > 0)
	{
		float3 reflectionVector = normalize(2 * NdotL * normal - lightDirection);
		
		specular = min(pow(saturate(dot(reflectionVector, input.ViewDirection)), SpecularPower), color.w);
		specular = get_color_contribution(SpecularColor, specular);
	}
	
	float4 output = diffuse + ambient + specular;
	
	if(FogEnabled)
	{
		output.rgb = apply_fog(FogColor, output, input.FogAmount);
	}
	
	return output;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 mainPS();
	}
}
