/*
% A fog shader with ambient light, spot light and texture mapping.
author: Alex Lemke
date: 4/20/12
*/

#include "include\Common.fxh"

float4x4 WorldViewProj : WORLDVIEWPROJECTION < string UIWidget = "none"; >;
float4x4 WorldInverseTranspose : WORLDINVERSETRANSPOSE < string UIWidget = "none"; >;
float4x4 World : WORLD < string UIWidget = "none"; >;
float4x4 ViewInverse : VIEWINVERSE < string UIWidget = "none"; >;

float4 AmbientColor : COLOR < string UIName = "Ambient Color"; string UIWidget = "Color"; > = {0.1, 0.1, 0.1, 0.1};

//Specular
float4 SpecularColor : SPECULAR < string UIName = "Specular Color"; string UIWidget = "Color"; > = {0.7, 0.7, 0.7, 1.0};
float SpecularPower : SpecularPower < string UIName = "Specular Power"; string UIWidget = "slider"; float UIMin = 1.0f, UIMax = 255.0f; float UIStep = 1.0f; >;

//Fog
float3 FogColor : COLOR < string UIWidget = "Fog Color"; > = {0.1, 0.1, 0.1};
float FogStart < string UIName = "Fog Start"; > = {10.0};
float FogEnd < string UIName = "Fog End"; > = {100.0};
bool FogEnabled < string UIName = "Fog Enabled"; > = false;

//Spot Light
float4 SpotLightColor : DIFFUSE < string UIName = "Spot Light Color"; string UIWidget = "Color"; string Object = "SpotLight0"; >;
float3 SpotLightPosition : POSITION < string UIName = "Spot Light Position"; string Object = "SpotLight0"; >;
float SpotLightRadius < string UIName = "Spot Light Radius"; string UIWidget = "slider"; float UIMin = 1.0f, UIMax = 255.0f; float UIStep = 1.0f; > = {10.0f};
float3 SpotLightLookAt : DIRECTION < string UIName = "Spot Light Direction"; string Object = "SpotLight0"; >;
float SpotLightInnerAngle < string UIName = "Spot Light Inner Angle"; string UIWidget = "slider"; float UIMin = 0.0f, UIMax = 1.0f; float UIStep = 0.01f; > = {0.75f};
float SpotLightOuterAngle < string UIName = "Spot Light Outer Angle"; string UIWidget = "slider"; float UIMin = 0.0f, UIMax = 0.5f; float UIStep = 0.01f; > = {0.25f};

//Texture
texture ColorTexture;
sampler ColorTextureSampler : register(s0) = sampler_state
{
	Texture = (ColorTexture);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

struct VS_IN
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
	float3 Normal 	: NORMAL;
};

struct VS_OUT
{
	float4 Position 		: POSITION0;
	float2 TexCoord 		: TEXCOORD0;
	float4 LightDirection 	: TEXCOORD1;
	float3 Normal 			: TEXCOORD2;
	float3 ViewDirection	: TEXCOORD3;
	float FogAmount 		: TEXCOORD4;	 
};

VS_OUT mainVS(VS_IN input)
{
	VS_OUT output;
	
	output.Position = mul(float4(input.Position.xyz, 1.0), WorldViewProj);
	output.TexCoord = get_corrected_texture_coord(input.TexCoord);
	output.Normal = mul(float4(input.Normal.xyz, 1.0), WorldInverseTranspose);
	
	float4 worldPosition = mul(float4(input.Position.xyz, 1.0), World);
	
	output.LightDirection = get_light_data(SpotLightPosition, worldPosition, SpotLightRadius);
	
	float3 cameraPosition = ViewInverse[3].xyz;
	
	output.ViewDirection = normalize(cameraPosition - worldPosition);
	output.FogAmount = get_fog_amount(cameraPosition, worldPosition, FogStart, FogEnd);
		
	return output;
}

float4 mainPS(VS_OUT input) : COLOR 
{
	//Normalizing Inputs
	float3 normal = normalize(input.Normal);
	float3 view = normalize(input.ViewDirection);
	float3 lightDirection = -normalize(input.LightDirection);
	
	float4 textureColor = tex2D(ColorTextureSampler, input.TexCoord);
	float NdotL = saturate(dot(lightDirection, normal));
	float3 halfVector = normalize(view + lightDirection);
	float NdotH = dot(normal, halfVector);
	
	//Calc lighting
	float4 lightCoefficients = lit(NdotL, NdotH, SpecularPower);
	
	float4 ambient = get_color_contribution(AmbientColor, textureColor);
	float4 diffuse = get_color_contribution(SpotLightColor, lightCoefficients.y * textureColor);
	float4 specular = get_color_contribution(SpecularColor, min(lightCoefficients.z, textureColor.w));

	float lightAngle = dot(-SpotLightLookAt, lightDirection);
	float spotFactor = 0.0f;
	if(lightAngle > 0.0f)
	{
		spotFactor = smoothstep(SpotLightOuterAngle, SpotLightInnerAngle, lightAngle);
	}
	
	float4 output = ambient + (spotFactor * get_color_contribution(SpotLightColor, (diffuse + specular)) * input.LightDirection.w);
	if(FogEnabled)
	{
		output.rgb = apply_fog(FogColor, output, input.FogAmount);
	}
	
	return output;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 mainPS();
	}
}
